<?php

namespace Drupal\inline_formatter_views_field\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\ResultRow;

/**
 * A handler to provide a HTML or Twig.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("inline_formatter_views_field")
 */
class InlineFormatterViewsField extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function usesGroupBy() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing -- to override the parent query.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['hide_alter_empty'] = ['default' => FALSE];
    $options['inline_formatter_field_display'] = ['default' => '<h1>Hello World!</h1>'];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $config = \Drupal::service('config.factory')->get('inline_formatter_field.settings');
    $form['#attached']['library'][] = 'inline_formatter_field/inline_formatter_display';
    // Add the ace editor settings to drupalSettings.
    $form['#attached']['drupalSettings']['inline_formatter_field']['ace_editor']['setting'] = [
      'ace_source' => $config->get('ace_source'),
      'theme' => $config->get('ace_theme'),
      'mode' => $config->get('ace_mode'),
      'options' => $config->get('extra_options'),
    ];
    // Ace Editor settings wrapper.
    $form['#prefix'] = '<div class="AceEditorFormatterSettings">';
    $form['#suffix'] = '</div>';
    $form['ace_options'] = [
      '#type' => 'details',
      '#title' => $this->t("Ace Editor Options"),
    ];
    // Ace Editor theme.
    $form['ace_options']['theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Theme'),
      '#options' => $config->get('available_themes'),
      '#default_value' => $config->get('ace_theme'),
      '#attributes' => [
        'class' => ['ace-theme-select'],
      ],
    ];
    // Ace Editor mode.
    $form['ace_options']['mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Mode'),
      '#options' => $config->get('available_modes'),
      '#default_value' => $config->get('ace_mode'),
      '#attributes' => [
        'class' => ['ace-mode-select'],
      ],
    ];
    // Ace Editor extra settings.
    $form['ace_options']['extra'] = [
      '#type' => 'fieldset',
      '#title' => $this->t("Extra Options"),
    ];
    $form['ace_options']['extra']['help'] = [
      '#markup' => $this->t('Options can be found on <a href=":url">Configuring Ace</a>.<br>Booleans should be entered "true" for true and "false" for false.', [
        ':url' => 'https://github.com/ajaxorg/ace/wiki/Configuring-Ace',
      ]),
    ];
    foreach ($config->get('extra_options') as $option => $value) {
      $form['ace_options']['extra'][$option] = [
        '#type' => 'textfield',
        '#title' => $option,
        '#default_value' => $value,
        '#attributes' => [
          'class' => ['ace-option-field'],
          'data-option-name' => $option,
        ],
      ];
    }
    // The actual textarea field.
    $form['inline_formatter_field_display'] = [
      '#title' => $this->t('HTML or Twig Format'),
      '#type' => 'textarea',
      '#default_value' => $this->options['inline_formatter_field_display'],
      '#description' => $this->t("Enter any HTML or Twig here along with token patterns.<br>Use 'opt + tab' and 'opt + shift + tab' to navigate through and back through the editor."),
      '#attributes' => [
        'class' => ['AceEditorTextarea'],
      ],
    ];
    // Ace Editor wrapper.
    $form['ace_editor'] = [
      '#type' => 'inline_template',
      '#template' => "<div class='EditorWrapper'><div id='AceEditor'></div><button class='fa fa-window-maximize ButtonSize'></button></div>",
    ];

    $form['help'] = $form['alter']['help'];
    unset($form['help']['#states']);
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $formatted_display = $this->options['inline_formatter_field_display'];
    $row_index = $values->index;
    $context = [];
    if (strpos($formatted_display, '{{') !== FALSE) {
      $fake_item = [
        'alter_text' => TRUE,
        'text' => $formatted_display,
      ];

      // Get tokens.
      if (isset($row_index) && isset($this->view->style_plugin->render_tokens[$row_index])) {
        $tokens = $this->view->style_plugin->render_tokens[$row_index];
      }
      else {
        // Get tokens from the last field.
        $last_field = end($this->view->field);
        if (isset($last_field->last_tokens)) {
          $tokens = $last_field->last_tokens;
        }
        else {
          $tokens = $last_field
            ->getRenderTokens($fake_item);
        }
      }
      // Update context.
      $context = $this->getContextFromTokens($tokens);
    }

    \Drupal::service('module_handler')->alter('inline_formatter_views_field_context', $context);

    return [
      '#type' => 'inline_template',
      '#template' => $formatted_display,
      '#context' => $context,
    ];
  }

  /**
   * Expands the given tokens into a context array.
   */
  protected function getContextFromTokens($tokens) {
    $twig_tokens = [];
    foreach ($tokens as $token => $replacement) {
      if (strpos($token, '{{') !== FALSE) {
        $token = trim(str_replace([
          '{{',
          '}}',
        ], '', $token));
      }
      if (strpos($token, '.') === FALSE) {
        // We need to validate tokens are valid Twig variables. Twig uses the
        // same variable naming rules as PHP.
        // @see http://php.net/manual/language.variables.basics.php
        assert(preg_match('/^[a-zA-Z_\\x7f-\\xff][a-zA-Z0-9_\\x7f-\\xff]*$/', $token) === 1, 'Tokens need to be valid Twig variables.');
        $twig_tokens[$token] = $replacement;
      }
      else {
        $parts = explode('.', $token);
        $top = array_shift($parts);
        assert(preg_match('/^[a-zA-Z_\\x7f-\\xff][a-zA-Z0-9_\\x7f-\\xff]*$/', $top) === 1, 'Tokens need to be valid Twig variables.');
        $token_array = [
          array_pop($parts) => $replacement,
        ];
        foreach (array_reverse($parts) as $key) {
          // The key could also be numeric (array index) so allow that.
          assert(is_numeric($key) || preg_match('/^[a-zA-Z_\\x7f-\\xff][a-zA-Z0-9_\\x7f-\\xff]*$/', $key) === 1, 'Tokens need to be valid Twig variables.');
          $token_array = [
            $key => $token_array,
          ];
        }
        if (!isset($twig_tokens[$top])) {
          $twig_tokens[$top] = [];
        }
        $twig_tokens[$top] += $token_array;
      }
    }
    return $twig_tokens;
  }

}
